local args = {...}
local uname=args[1]

-- The URI table, a table with URI's the user is allowed to visit.
local urit={}

local userDb = bd.db.user.load()
local constrDb = bd.db.drvcnstr.load()

if userDb and constrDb and userDb.v[uname] then
   local rt = {} -- Role Table

   -- Match roles in Roles Table rt with a list of roles.
   function matchRoles(list)
      if #list == 0 then
         return true -- Empty roles list implies all roles (wildcard role)
      end
      for _,v in ipairs(list) do
         if rt[v] then
            return true
         end
      end
      return false
   end

   -- Translate role list to table for faster lookups.
   local inAllRoles=true -- User is in all roles if roles list is empty.
   -- Translate
   for _,v in ipairs(userDb.v[uname].roles) do
      rt[v]=true;
      inAllRoles=false
   end

   -- Populate the uri table with uri's the user is allowed to access.
   local user = userDb.v[uname]
   for _,v in pairs(constrDb.v) do
      if inAllRoles or matchRoles(v.roles) then
         -- Fetch all the URI's
         for _,u in ipairs(v.urls) do
            local uri=string.gsub(u,"(.-)%*$", "%1") -- Strip off any /*
            if #uri > 0 and (#v.methods ~= 1 or v.methods[1] ~= "DISABLED") then
               local mt -- A table with all methods allowed
               if urit[uri] then -- Append methods
                  mt = urit[uri]
               else -- New methods
                  mt = {}
               end
               for _,m in ipairs(v.methods) do
                  mt[m]=true;
               end
               urit[uri]=mt
            end
         end
      end
   end
end

-- Takes a table of methods and translates to a string
function accessRights(mt)
   local read = mt["PROPFIND"] and mt["GET"] and "read" or "";
   local write = mt["POST"] and mt["PUT"] and "write" or "";
   return read.." - "..write
end

return urit,accessRights
