<?lsp

local fmt=string.format
local io=app.docsIo

local function doErr(msg, fatal)
   trace("CMS imgupload:", msg)
   if fatal then error(msg,2) end
   return msg
end

local function getCmsT() return app.getCmsTab(request:session()) end

local function getCfgT() return getCmsT().imgUplCfg end

local function quote(s) return s:gsub('"','""') end

local function getAlbumKey(cfg)
   local key
   local q=fmt('SELECT key FROM PhotoAlbumNames WHERE name="%s"',
               cfg.albumtitle)
   local cur=conn:execute(q)
   if cur then
      key = cur:fetch()
      cur:close()
   end
   return key
end


local function save(list)
   local ok,err
   conn:setautocommit("IMMEDIATE", 1000)
   local cfg=getCfgT()
   local albumKey = getAlbumKey(cfg)
   if not albumKey then
      local q=fmt('INSERT INTO PhotoAlbumNames (name) values("%s")',
                  cfg.albumtitle)
      ok,err=conn:execute(q)
      albumKey=conn:getlastautoid()
   else
      local q=fmt('DELETE FROM PhotoAlbums WHERE album=%u', albumKey)
      ok,err=conn:execute(q)
   end
   if not err then
      local fmts='INSERT INTO PhotoAlbums (album,img,info) values(%u,"%s","%s")'
      for _,n in pairs(list) do
         local info=n.info:gsub('(<.->)',''):gsub('(&%a-;)',''):gsub('"','')
         ok,err=conn:execute(fmt(fmts, albumKey, quote(app.thumb2ImgName(n.img)), info))
         if err then break end
      end
      if err then conn:rollback() else conn:commit() end
   end
   if err then doErr(err) end
   return "ok"
end


local function remove(images)
   local ok,err
   conn:setautocommit("IMMEDIATE", 1000)
   local albumKey = getAlbumKey(getCfgT())
   for _,thumb in pairs(images) do
      local name = app.thumb2ImgName(thumb)
      if albumKey then
         local q=fmt('DELETE FROM PhotoAlbums WHERE album=%u AND img="%s"',albumKey,name)
         ok,err=conn:execute(q)
      end
      if not err then
         ok,err=io:remove(thumb)
         if ok then
            ok,err=io:remove(name)
            thumb=name
         end
      end
      if err then
         conn:rollback()
         return doErr(fmt("Remove %s: %s",thumb,err))
      end
   end
   conn:commit()
   return "ok"
end

local function list(images)
   if gd then
      return app.io:loadfile(pathname:match("(.-)[^/]+$")..".albumlist.lua")(app)
   end
   return "No GD"
end

local function createhtml(album)
   local t={}
   local q=fmt([[SELECT PhotoAlbums.img,PhotoAlbums.info FROM PhotoAlbumNames,PhotoAlbums
                 WHERE PhotoAlbumNames.name="%s" AND
                 PhotoAlbumNames.key = PhotoAlbums.album
                 ORDER BY PhotoAlbums.key]], album)
   local cur=conn:execute(q)
   if cur then
      local tinsert = table.insert
      tinsert(t,fmt('\n\n<div id="%s" class="LoadPhotoAlbum"><h1>%s</h1><div class="PhotoAlbum"><ul>\n',album,album))
      local thumb = app.imgName2Thumb
      local img,info = cur:fetch()
      while img do
         tinsert(t,fmt('<li><span><a href="%s" title="%s"><img src="%s" alt="" /></a></span></li>\n',
                       img,info,thumb(img)))
         img,info = cur:fetch()
      end
      cur:close()
      tinsert(t,'</ul></div></div>\n\n')
   end
   return table.concat(t)
end

local service = jsonrpc.new("Album",
                               {
                                  album={
                                     remove=remove,
                                     save=save,
                                     list=list,
                                     createhtml=createhtml,
                                  }
                               })
response:reset()
env,conn = app.openDb()
service:execute(request, response)
app.closeDb(env,conn)
?>
