<?lsp

local fmt = string.format
local gsub=string.gsub
local function toRssDate(time)
   return os.date("%a, %d %b %Y %H:%M:%S GMT",time)
end

local function emitRssItem(title,url,description,date)
   --Strip HTML tags
   title = gsub(title, "(]].->)", "]]")
   description = gsub(description, "(]].->)", "]]")
   return fmt('<item><title><![CDATA[%s]]></title><link>%s</link><description><![CDATA[%s]]></description><pubDate>%s</pubDate></item>',
     title,
     url,
     description,
     toRssDate(date))
end

local env,conn = app.openDb()

local cur = conn:execute('SELECT k, date, name, email, message FROM feedback ORDER BY date DESC')
local key, date, name, email, message=cur:fetch()

local function close()
   cur:close()
   app.closeDb(env,conn)
end

local acceptsDeflate
do
   local ae = request:header("Accept-Encoding")
   acceptsDeflate = ae and ae:find("deflate") and true or false
end


local pubDate = date
if not pubDate then
   if not app.rssDate then
      app.rssDate = os.time()
   end
   pubDate=app.rssDate
end

local lastModified=tonumber(pubDate)

if not request:checktime(lastModified) then
   close()
   return
end

-- Remove default LSP headers such as Cache-Control, etc.
response:reset()
if acceptsDeflate then
   response:setheader("Content-Encoding", "deflate")
end
response:setdateheader("Last-Modified", lastModified)
pubDate=toRssDate(pubDate)
response:setcontenttype"text/xml;charset=utf-8"

-- Check for cached version on disk

-- Opens in read mode if date is set. Returns nil if not found or date expired.
local function openCached(date)
   local bio = bd.baseio
   local cn = app.getCacheDir"messages.rss"
   if date then
      local st = bio:stat(cn)
      if st and st.mtime > tonumber(date) then
         return bio:open(cn, "rb") 
      end
   else
      return bio:open(cn, "wb") 
   end
   return nil
end

if date and acceptsDeflate then
   local fp = openCached(date)
   if fp then
      local data = fp:read"*a"
      fp:close()
      response:setheader("Content-Length",#data)
      response:send(data)
      return
   end
end


-- Page content is inserted into this table and not to the response object
local deflateTab={}
function rssEmit(data)
   table.insert(deflateTab,data)
end
_emit = rssEmit


local domain = request:domain()
local url=fmt("http://%s/private/manage/messages.lsp",domain)
?>
<?xml version="1.0"?> 
<rss version="2.0">
<channel>
<title>Messages from <?lsp=domain?></title> 
<link>http://<?lsp=domain?></link> 
<description>Subscribe to this feed if you like to get the latest feedback response messages.</description> 
<pubDate><?lsp=pubDate?></pubDate> 
<lastBuildDate><?lsp=pubDate?></lastBuildDate> 
<generator>FuguHub</generator> 
<ttl>240</ttl> 
<?lsp
while key do
   rssEmit(emitRssItem(fmt("%s %s",name,email),
                       fmt("%s?key=%s",url,key),message,date))
   key, date, name, email, message=cur:fetch()
end
close()
rssEmit("</channel></rss>")

if acceptsDeflate then
   local data = ba.deflate(deflateTab)
   response:setcontentlength(#data)
   response:send(data)
   local fp = openCached()
   if fp then
      fp:write(data)
      fp:close()
   end
else
   local data = table.concat(deflateTab)
   response:setcontentlength(#data)
   response:send(data)
end
?>
