<?lsp

local session = request:session()
if not session or not session.testPortMapping then
   response:sendredirect"SearchUPnP.lsp"
   return
end
local wanHttp, wanHttps = bd.ddns().wanPorts()

local fmt=string.format

-- The JavaScript code we use requires that we pipe the data via the
-- FuguHub server to disable the cross browser prevention logic
-- in the browser.
-- We can now access any server by using the URI:
-- /Config-Wizard/protected/_r_/domainname/path
-- The redirector object is created in the session object and is
-- therefore automatically removed when the session expires.
if not session.redirector then
   session.redirector = ba.create.redirector("protected/_r_")
   app.dir:insert(session.redirector)
end

headerdata=fmt([[
<script src="../help.js"></script>
<script type="text/javascript" src="/rtl/jquery.js"></script>
<script>
var testResult=null;
var wanAddr;
function setPfConsole(text) // Port Forward Console
{
   $("#pfConsole").html(text);
};

function printTestResult(httpOk, httpsOk)
{
   var msg;
   if(httpOk == "OK" && httpsOk == "OK")
   {
      testResult=true;
      msg="<p>Congratulations, your server is accessible externally. Our online server FuguHub.com has verified, by connecting to your WAN address "+
         wanAddr+
         ", that the standard HTTP and secure HTTP (HTTPS) are working -- i.e. port 80 and port 443 are working. Your Port Mapping is properly configured.</p><p>Click the arrow to continue.</p>";
      $("#TestButton").fadeOut("slow");
      setPfConsole(msg);
   }
   else {
      if(httpOk != "OK" && httpsOk != "OK") {
         testResult=false;
         msg="<p>Your server is not accessible externally.</p>";
      }
      else {
         testResult=true;
         msg="<p>Your server is accessible externally using " +
            (httpOk == "OK" ? "HTTP" : "HTTPS") +
            " but not using "+
            (httpsOk == "OK" ? "HTTP" : "HTTPS") + ".</p>";
      }
      function status(port) {return port == "OK" ? "true" : "false"; };
      var exmsg='<h1><a href="../PortMappingFailed/?http='+status(httpOk)+'&https='+status(httpsOk)+'">Fix the problem</a></h1>';
      setPfConsole('<h1 style="color:red">Test Failed</h1>'+msg+exmsg);
   }
}


function testPF()
{
   $("#WanInfo").fadeOut("slow");
   setPfConsole("<p><b>Running test...<br/>Please wait.</b></p>")

   $.ajax({
     type: "POST",
     url: "_r_/FuguHub.com/bd/testserver.lsp",
     data: "http=%d&https=%d",
     complete: function(x){
        if(x.status == 200) {
            var httpOk = x.getResponseHeader("http");
            var httpsOk = x.getResponseHeader("https");
            printTestResult(httpOk, httpsOk)
         }
         else {
            setPfConsole("<p>Test failed: status="+status+"</p>")
         }
     }
   });
}

function showWan(myip)
{
   wanAddr=myip;
   $("#WanAddr").html(myip+", and your WAN URL is: http://"+myip);
   $("#WanMsg").fadeIn("slow");
};


//Code for dynamically adding the WAN IP address to the document.
onload = function()
{
   $("#NextButton").click(function() {
      if(testResult) return true;
      var msg="Are you sure you want to continue?\n";
      msg += testResult == false ? "The port mapping test failed" : "You have not run the port mapping test";
      msg +="\nPress the 'Cancel' button if you like to continue with the port mapping test.";
      return confirm(msg);
   });
   $.ajax({
     type: "GET",
     url: "_r_/FuguHub.com/bd/myip.lsp",
     complete: function(x){
        if(x.status == 200) {
           setPfConsole("");
           showWan(x.getResponseHeader("myip"));
        }
        else {
           var msg="<p style='color:red;font-weight:bold'>Error: The server is unable to connect to the Internet. This could indicate a problem with your network settings such as the default gateway settings or DNS settings.</p>"
              if(window.warning)
                 msg = msg + "<p>Did you fix the following problem?</p>"+warning;
           setPfConsole(msg);
           $("#TryAgain").fadeIn("slow");
        }
     }
   });
};
</script> 
]], wanHttp, wanHttps)
title="Test Port Mapping"
response:include"../wizard/.header.lsp"
if session.warning then
   print(fmt('<script>var warning="%s";</script>', session.warning));
end
?>
<h1><?lsp=title?></h1> 
<p style="float:right"><a id = "NextButton" href="<?lsp=session.upnpMapping and "ddnsIntro.lsp" or "../intro/"?>" title="Next"><img src="../images/next.png" border="0" /></a></p>
<div id="WanMsg" style="display:none">
<p><br/></p>
<p id="WanInfo">Your WAN IP address is: <span id="WanAddr"></span>. Our online server FuguHub.com can automatically test if you have properly configured port forwarding by connecting to your WAN URL. Click the button below to test your port forwarding configuration.<br/><br/>
Note that the WAN URL will most likely not function from your local computer or any other computer on your LAN. Computers on your LAN must use the local IP address, not the external WAN IP address.</p>

<p><br/></p>
<p id="TestButton" style="text-align:center" style="height:90px">
<input type="button" value="Test Port Mapping" onclick="testPF()" />
</p>
</div>
<div id="pfConsole">
<p style="text-align:center"><img src="/rtl/wfm-album/loading.gif" alt=""/></p>
</div>
<h3 id="TryAgain" style="display:none"><a href='TestPortMapping.lsp'>Try again</a></h3>

<?lsp
showAbort=true
response:include"../wizard/.footer.lsp"
?>
