<?lsp


local gmatch = string.gmatch
local tinsert = table.insert
local fmt=string.format

local emsg
local data={}
local maxsize=1024*1024
local methodAction={
   POST=function()
      local size=0
      request:multipart({
         beginfile=function() end, -- Required, but not used.
         filedata=function(chunk)
            size=size+#chunk
            if size > maxsize then
               emsg="File too large"
            else
               table.insert(data,chunk)
            end
         end,
         error=function(e)
            emsg=e
         end
      })
   end,
   PUT=function()
      local size=0
      for chunk in request:rawrdr() do
         size=size+#chunk
         if size > maxsize then
            emsg="File too large"
            break
         end
         table.insert(data,chunk)
      end
   end
}
methodAction=methodAction[request:method()]
local newUsers
if not methodAction then
   newUsers=request:session(true).newUsers
end


-- The extra header we need ".header" to add.
headerdata=[[
<link href="style.css" rel="stylesheet" type="text/css" />
<link rel="stylesheet" type="text/css" href="../../fstab.css" />
<style>
.center {margin-left: auto; margin-right: auto;}
#info {
display:none;
float:right;
margin:0 15px 0 15px;
width:200px;
padding:10px;
}

#info p {font-weight:bold;}

</style>
<script type='text/javascript' src='/rtl/wfm/wfm.js'></script>
<script type='text/javascript'>
$(function() {
   function checkExt(name) {
      var filter=true;
      try {
         var regexp=/^.*\.([^\.]*)$/m;
         var x=name.match(regexp);
         var ext=x[1].toLowerCase();
         filter=ext!="csv";
      }
      catch(e){alert(e);}
      if(filter) alert("You are attempting to upload a file with a non supported file extension!");
      return filter;
   }
   try {
      var xhr = new XMLHttpRequest();
      xhr.upload.addEventListener("progress", function(){},false);
      $("#info").show();
      window.dropURL=window.location.href; //For WFM drag & drop
      window.dropFilter=checkExt;
   }
   catch(e) {}
   $("#introduction").hide();
   $("img.lhelp").click(function() {$(this).hide();$("#introduction").fadeIn();});
});
</script>
]]
 
title="Menu: Administrator Panel: Import Users"
response:include"/rtl/.header.lsp"
if not newUsers then
?>
<img class="lhelp" src="/rtl/bindows/html/images/questionmark.32.gif"/></a>
<?lsp end ?>
<h1>Import Users</h1>
<?lsp
if methodAction then
   local users=bd.db.user.load()
   local usersV=users.v
   local trim=bd.trim

   local function setName(user, name)
      user.name=trim(name)
   end

   local function setPwd(user, pwd)
      user.pwd=trim(pwd)
   end

   local function setRoles(user, groups)
      local roles={}
      for group in gmatch(groups, "([^%s]+)") do
         tinsert(roles,trim(group))
      end
      user.roles=roles
   end

   local function setInactive(user, inactive)
      inactive = tonumber(inactive)
      if inactive then
         user.inactive=inactive
      end
   end

   local function setMaxUsers(user, maxUsers)
      maxUsers = tonumber(maxUsers)
      if maxUsers then
         user.maxUsers=maxUsers
      end
   end

   local function setRecycle(user, recycle)
      user.recycle = trim(recycle) == "true"
   end

   local s = request:session(true)
   local action={setName,setPwd,setRoles,setInactive,setMaxUsers,setRecycle}
   local newUsers={}
   local t = os.time()
   -- Parse CSV file
   local lineCnt=1
   methodAction()
   local ok,err=pcall(function()
      if emsg then error("Upload failed: "..emsg) end
      data=table.concat(data)
      local csv,err=bd.csv(data,true)
      if not csv then error(err) end
      for _,line in ipairs(csv) do
         -- User requires: name, pwd, roles: The following are default for opt vals.
         local user={
            inactive=1200,
            maxUsers=3,
            recycle=true,
            t=t,
         }
         local maxLen=#action
         for i,x in ipairs(line) do
            if i <= maxLen then
               action[i](user,x)
            end
         end
         if user.name and user.pwd and user.roles and #user.roles > 0 and
            not usersV[user.name] and not newUsers[user.name] then
            newUsers[user.name]=user
         end
         lineCnt = lineCnt+1
      end
   end)
   if not ok then
      response:reset"buffer"
      if request:method() == "PUT" then -- Drag and drop. See dragdrop.txt.
         response:write(ba.json.encode{err=err})
      else
         response:include"/rtl/.header.lsp"
         response:write('<p style="color:red">',err,
                        "</p><p>Press the browser's back button.</p>")
         response:include".footer.lsp"
      end
      return
   end
   if request:method() == "PUT" then
      response:write(ba.json.encode{execjs="location.reload(true)"})
   end
   s.newUsers = newUsers
end
if newUsers then
   local data=request:data()
   if not data.save then
      if data.discard then
         request:session().newUsers=nil
         response:sendredirect("")
      end
      local function w(v) response:write("<td>",v,"</td>") end
      local function wt(t) for _,v in ipairs(t)do w(v) end end
      response:write"<div id='fstab'><table><thead><tr><th>Name</th><th>Roles</th><th>Inactive</th><th>Max Login</th><th>Recycle</th></tr></thead><tbody>"
      for name,user in pairs(newUsers) do
         response:write"<tr>"
         w(user.name) w(table.concat(user.roles,"<br/>")) w(user.inactive)
         w(user.maxUsers) w(user.recycle and "true" or "false")
         response:write"</tr>"
      end
      response:write"</tbody></table></div>"
?>
<br/>
<div class="center" style="width:50px">
<form method="get">
<table><tr>
<td><input type="submit" name="save" value="Save"></input></td>
<td><input type="submit" name="discard" value="Discard"></input></td>
</tr></table>
</form>
</div>
<?lsp
   else
      local s = request:session()
      local newUsers = s.newUsers
      s.newUsers=nil
      local users=bd.db.user.load()
      local usersV=users.v
      for name,user in pairs(newUsers) do
         usersV[name] = user
      end
      bd.db.user.save(users)
?>
<p>Users saved and activated.</p>
<p><a href="<?lsp=request:url()?>">continue</a></p>
<?lsp end else ?>
<div class="center" style="width:350px;padding:2em">
<form method="post" enctype="multipart/form-data">
<input type='file' size='40' name='CSV'/><br/>
<input type="submit" value="Upload"/>
</form>
</div>
<div id="introduction">
<div class="roundedbox" id="info">
<img src="../../images/dropbox64.png"/ style="float:right">
<p>Your browser supports drag and drop upload!</p>
<p>Drag and drop the CSV file into this browser window.</p>
</div>
<p>The batch user importer makes it possible to import many users by uploading a file with all users you want added to the FuguHub user database.</p>


<p>The format for the file to upload using the batch user importer should be a CSV  (<a href="http://en.wikipedia.org/wiki/Comma-separated_values" target="_blank">comma-separated value</a>) file with the following:<br/>
<b>username, password, roles</b><br/>
where roles is: role1 role2 etc
</p>

<p>In addition, the following optional fields may be added:<br/>
<b>InactiveTimeInSeconds, maxConcurrentLogins, recycleConnections(true/false)</b><br/>
Default for the above is: 1200, 3, true
</p>


<p style="font-size:140%;font-weight:bold">CSV Example File:</p>
<pre style="border:1px solid;width:450px;padding:5px">
mark,   temp123456, users admin, 2000, 5, true
oliver, temp123456, users,       600
</pre>

<p>Make sure your file is saved in the UTF-8 format if the file you upload contains non English letters. A CSV file can be created by exporting an Excel Spreadsheet.</p>
</div>
<?lsp
end
response:write'<div id="overlaymask"></div>'
response:include".footer.lsp"
?>
