<?lsp

local fmt=string.format
local tinsert = table.insert

local aroles=bd.db.roles.load().v -- All roles

local toInt
if math and math.ceil then toInt=function(x) if(x) then return math.ceil(x) end end
else toInt=function(x) return x end
end

local function getChecked(attr, val)
   if attr == val then
      return " checked"
   end
   return ""
end

local list2tab = bd.list2tab
local tab2list = bd.tab2list
 
local function emitRadio(group, val, attr)
   response:write('<input type="radio" name="',group,'" value="',
                  val and 'true' or 'false','"',
                  getChecked(attr,val),'/>')
end

local function emitOptions(list, excludeList)
   local ex=list2tab(excludeList)
   table.sort(list)
   for _,v in ipairs(list) do
      if not ex[v] then
         response:write('<option value="',ba.urlencode(v),'">',v,'</option>')
      end
   end
end

local users
local user
local data=request:data()
if data.name then
   if data.name ~= bd.xssfilter(data.name) then
      response:sendredirect"users.lsp"
   end
   users = bd.db.user.load().v
   user = users[data.name]
end
for k,v in pairs(data) do data[k] = v:gsub("[<>]","") end


local saveData
local baseTitle="Menu: Administrator Panel: "
if user then
   addUserTxt="Edit User"
   title = baseTitle..addUserTxt
else
   title = baseTitle.."Add User"
   user={roles={"user"}}
end
if request:method() == "POST" and data.name then
   local adminname=request:user()
   user.pwd=data.pwd
   local maxUsers=toInt(tonumber(data.maxUsers))
   user.recycle=data.recycle == "true" and true or false
   local inactive=toInt(tonumber(data.inactive))
   if maxUsers and maxUsers >= 0 then
      if maxUsers == 0 and adminname == data.name then
         -- Cannot disable admin
         user.maxUsers = 1
      else
         user.maxUsers = maxUsers
      end
   end
   if inactive and inactive > 0 then user.inactive = inactive * 60 end
   user.info=data.info
   user.pwdl=data.pwdl == "true" and true or false
   local ar={}
   local ur={}
   local roles=list2tab(user.roles)
   for n,v in request:datapairs() do
      if n == "ar" then
         ar[v]=true
      elseif n == "ur" then
         ur[v]=true
      end
   end
   for k,_ in pairs(ur) do roles[k]=nil end
   for k,_ in pairs(ar) do roles[k]=true end
   if adminname == data.name then
      -- Must be in admin group if groups not empty.
      for k,_ in pairs(roles) do
         roles['admin']=true
         break
      end
   end
   user.roles=tab2list(roles)
   saveData = true
end

local function setDefUserAttr(name, val)
   if not user[name] and user[name] ~= false then user[name] = val end
end
setDefUserAttr("pwd","")
setDefUserAttr("maxUsers",3)
setDefUserAttr("recycle",true)
setDefUserAttr("inactive",1200)
setDefUserAttr("roles",{"user"})
setDefUserAttr("info","")
setDefUserAttr("t",os.time())
setDefUserAttr("pwdl",false)
local inactive = toInt(user.inactive/60)

if saveData then
   user.name=data.name
   users[data.name]=user
   bd.db.user.save({v=users})
   response:sendredirect(fmt("%s?name=%s",request:url(),ba.urlencode(data.name)))
   return
end

-- The extra header we need ".header" to add.
headerdata=[[
<link href="style.css" rel="stylesheet" type="text/css" />
<style>
 input,.talignr {text-align:right;}
 .fw {width:140px;}
select,textarea {width:250px;margin:5px}
fieldset { float:left;}
form {margin-left: auto;margin-right: auto;width:600px;}
td {padding:3px}
</style>
]]
response:include".header.lsp"

-- name pwd maxUsers recycle inactive roles info t pwdl
-- t: creation time, pwdl: Password locked (bool)

?>
<form class="autow roundedbox" style="width:600px" method="post" autocomplete="off">
<?lsp emitHelpBut("manageuser") ?>
<table><tr><td>
<table><tr>
<td><table>
  <tr>
    <td><b>Name:</b></td>
    <td>
      <?lsp
        if user.name then response:write("<div class='talignr'>",user.name,"</div>")
        else response:write'<input "class="fw" type="text" name="name"/>'
        end
      ?>
    </td>
  </tr>
  <tr>
    <td><b>Password:</b></td>
    <td><input class="fw" type="password" name="pwd" value="<?lsp=user.pwd?>" autocomplete="off" /></td>
  </tr>
  <tr>
    <td><b>Inactive Timeout:</b></td>
    <td><input class="fw" type="text" name="inactive" value="<?lsp=inactive?>"/></td>
  </tr>
  <tr>
    <td><b>Max Logons:</b></td>
    <td><input class="fw" type="text" name="maxUsers" value="<?lsp=user.maxUsers?>"/></td>
  </tr>
  <tr>
    <td><b>Recycle Sessions:</b></td>
    <td>
      <?lsp
        emitRadio("recycle", true, user.recycle) response:write(" Yes ")
        emitRadio("recycle", false, user.recycle) response:write(" No")
      ?>
    </td>
  </tr>
  <tr>
    <td><b>Password Locked:</b></td>
    <td>
      <?lsp
        emitRadio("pwdl", true, user.pwdl) response:write(" Yes ")
        emitRadio("pwdl", false, user.pwdl) response:write(" No")
      ?>
    </td>
  </tr>
</table></td>
</tr>

<tr>
<td><fieldset><legend><b> User's Roles</b></legend>
   <select <?lsp=#user.roles==0 and 'id="superuser"' or ''?> name="ur" multiple="multiple" size="10">
   <?lsp emitOptions(user.roles) ?>
</select></fieldset>
</td>
</tr>
<tr><td>
<fieldset><legend><b> User Information </b></legend>
<textarea rows="7" name="info"><?lsp=user.info?></textarea>
</fieldset>


</td>
</tr></table>
</td>
<td><fieldset><legend><b> All Roles </b></legend>
   <select name="ar" multiple="multiple" size="32">
   <?lsp emitOptions(aroles,user.roles) ?>
</select></fieldset></td>
<td>
</td>
</tr></table>
<input type="submit" value="Save"/>
</form>

<?lsp response:include".footer.lsp" ?>
