<?lsp

local fmt=string.format


-- The extra header we need ".header" to add.
headerdata=[[
<link rel="stylesheet" type="text/css" href="../../fstab.css" />
<style>
.center {margin-left: auto; margin-right: auto;}
#filter input {width:100px;}
a.button{
color:black;
background-color:#eeeeee;
padding:5px 10px 5px 10px;
border:1px solid;
}
input.cbrmarg { margin-right:1em}
</style>
<script type='text/javascript' src='/rtl/wfm/wfm.js'></script>
<script type='text/javascript'>
$(function() {
   $("#selall").click(function() {
      $("#fstab input").attr("checked","checked");
      $(this).blur();
   });
});
</script>
]]
 
title="Menu: Administrator Panel: All Users"
response:include"/rtl/.header.lsp"


local trim=bd.trim

local data=request:data()
local function filter()
   return true
end
local function mkDateFilter()
   if data.fsdate and data.fedate then
      local function getDate(dstr)
         local y,m,d = dstr:match'(%d%d%d%d)-(%d%d)-(%d%d)'
         if y and m and d then
            local t = os.time{year=y,month=m,day=d}
            if t then 
               return t
            end
         end
         return nil,""
      end
      local s = getDate(data.fsdate)
      local e = getDate(data.fedate)
      if s and e then
         local function dFilter(user)
            local t=user.t
            if type(t) == "number" and s <= t and t <= e then
               return true
            end
            return false
         end
         filter=dFilter
         return
      end
   end
   data.fsdate,data.fedate="",""
end

local function mkRoleFilter()
   if data.frole then
      data.frole = trim(data.frole)
      if #data.frole > 0 then
         local nextFilter=filter
         local function roleFilter(user)
            local fr=data.frole
            for _,r in ipairs(user.roles) do
               if r == fr then
                  return nextFilter(user)
               end
            end
            return false
         end
         filter=roleFilter
         return
      end
   end
   data.frole=""
end

local function mkNameFilter()
   if data.fname then
      data.fname = trim(data.fname)
      if #data.fname > 0 then
         local nextFilter=filter
         local function nameFilter(user)
            return user.name and
               user.name:find(data.fname) and
               nextFilter(user) or false
         end
         filter=nameFilter
         return
      end
   end
   data.fname=""
end

mkDateFilter()
mkRoleFilter()
mkNameFilter()

local bio = bd.baseio
local session=request:session()
local users
local sortedUsers

local function mkSortedUsers()
   sortedUsers = {}
   local tins = table.insert
   for name,_ in pairs(users.v) do
      tins(sortedUsers, name)
   end
   table.sort(sortedUsers)
end

if session.users and data.cache then
   users = session.users
   sortedUsers = session.sortedUsers
else
   users = bd.db.user.load()
   mkSortedUsers()
   session.users = users
   session.sortedUsers=sortedUsers
end
local usersV = users.v


local function emitNav(lname, start)
   local k={"fname","frole","fsdate","fedate"}
   local vals={data.fname,data.frole,data.fsdate,data.fedate}
   local d={}
   for i,v in ipairs(vals) do
      if #v > 0 then
         table.insert(d, fmt("%s=%s",k[i],v))
      end
   end
   table.insert(d, fmt("start=%u",start))
   response:write('<a class="button" href="users.lsp?',table.concat(d, "&"),
                  '&cache=true">',lname,'</a>')
end

local respT={}
local function w(...)
   for _,v in ipairs{...} do
      table.insert(respT,v)
   end
end

local function wt(...)
   table.insert(respT,"<td>")
   w(...)
   table.insert(respT,"</td>")
end

if request:method() == "POST" and data.name then
   local started
   local uname=request:user()
   for name,value in request:datapairs() do
      if name == "name" and value ~= uname then
         if not started then
            response:write"Deleted: "
            started=true
         end
         response:write(value," ")
         usersV[value]=nil
      end
   end
   if started then
      bd.db.user.save(users)
      mkSortedUsers()
   end
end

local start=data.start and tonumber(data.start) or 1
local cnt=0
local i=start
while i <= #sortedUsers do
   local name = sortedUsers[i]
   local user=usersV[name]
   i=i+1
   if user and filter(user) then
      cnt = cnt+1
      if cnt == 100 then
         break
      end
      w('<tr><td><input type="checkbox" class="cbrmarg" name="name" value="',
        name,'"><a href="user.lsp?name=',ba.urlencode(name),'">',name,'</a></input></td>')
      wt(table.concat(user.roles,"<br/>"))
      if type(user.t) == "number" then
         wt(os.date('%Y-%m-%d',user.t))
      else
         wt('')
      end
      w"</tr>"
   end
end
?>
<table style="width:100%<?lsp=#sortedUsers<100 and ';display:none' or ''?>"><tr><td style="width:50px">
<?lsp
if start > 1 then
   local x=start-100
   emitNav("prev", x > 1 and x or 1)
end
?>
</td><td>
<form method="get">
<fieldset id="filter"><legend>Filters:</legend>
<table>
<tr>
<td>Name:</td><td><input type="text" name="fname" value="<?lsp=data.fname?>"/></td>
<td>Start date:</td><td><input type="text" name="fsdate" value="<?lsp=data.fsdate?>"/></td>
<td></td>
</tr>
<tr>
<td>Role:</td><td><input type="text" name="frole" value="<?lsp=data.frole?>"/></td>
<td>End date:</td><td><input type="text" name="fedate"  value="<?lsp=data.fedate?>"/></td>
<td><input type="submit" value="Set"/></td>
</tr>
</table>
</fieldset>
<input type="hidden" name="start" value="<?lsp=start?>" />
<input type="hidden" name="cache" value="true" />
</form>
</td><td style="width:50px">
<?lsp
if i < #sortedUsers then
   emitNav("next", i)
end
?>
</td></tr></table>
<?lsp
if cnt > 0 then
   response:write'<form method="post"><div id="fstab"><table><thead><tr><th>Name</th><th>Roles</th><th>Date Added</th></tr></thead><tbody>'
   response:write(table.concat(respT))
?>
</tbody>
</table>
<input type="hidden" name="start" value="',start,'" />
<input type="hidden" name="cache" value="true" />
<br/>
<input type="submit" value="Delete selected users"/>
</div>
</form>
<button type="button" id="selall" style="margin-top:5px">Select all</button>
<?lsp
end
response:include".footer.lsp"
?>

