<?lsp

local tinsert=table.insert
local fmt=string.format

local allmethods={
   "GET","POST","PUT","PROPFIND","PROPPATCH",
   "MKCOL","DELETE","MOVE","COPY"
}


-- Collect all url encoded data.
-- "croles" and "aroles" are from the select:option multisets
-- croles: Constraint roles, aroles: All roles
local data={croles={},aroles={}}
do
   local action={
      croles=function(val) tinsert(data.croles, val) end,
      aroles=function(val) tinsert(data.aroles, val) end,
      constr=function() end
   }
   for key,val in request:datapairs() do
      val = val:gsub("^%s*(.-)%s*$", "%1") -- trim
      if #val ~= 0 then
         if not data[key] then
            data[key]=val:gsub("[<>]","")
         else
            local action=action[key]
            if not action then
               response:senderror(403)
               return
            end
            action(val)
         end
      end
   end
end

local list2tab = bd.list2tab
local tab2list = bd.tab2list

local function sendredirect(key,val)
   local url=request:url()
   if key then
      response:sendredirect(fmt("%s?%s=%s",url, key, val and ba.urlencode(val) or ""))
   else
      response:sendredirect(url)
   end
end

local function sendredirect2constr()
   if data.constr then
      sendredirect("constr", data.constr)
   else
      sendredirect()
   end
end

local function emitCheckbox(name, isChecked)
   response:write('<input type="checkbox" name="',name,'" id="',name,'"',
                  isChecked and " checked" or "",'/>')
end

local function emitOptions(list, excludeList, selname)
   local ex=list2tab(excludeList)
   table.sort(list)
   for _,v in ipairs(list) do
      if not ex[v] then
         response:write('<option ',selname == v and 'selected="selected" ' or '',
                        'value="',ba.urlencode(v),'">',v,'</option>')
      end
   end
end

local constrs=bd.db.drvcnstr.load().v -- All constraints

 -- Handle Constraint Submit Button first, if clicked
if data.constrsub then
   if data.constr then -- name
      if data.constrsub == "Delete" then
         constrs[data.constr]=nil
         bd.db.drvcnstr.save({v=constrs})
      else
         sendredirect2constr()
         return
      end
   end
   sendredirect()
   return
end

-- Extract data required for the various page click and submit actions
local aroles=bd.db.roles.load().v -- All roles
local constrnames=tab2list(constrs) -- Create a list of constr. names
local selconstr=constrs[data.constr] -- The selected constraint, if any
local session = request:session(true)

if data.basepath then
   session.constrBasepath=data.basepath
elseif not session.constrBasepath then
   session.constrBasepath="/"
end

local sconstr = session.constr -- The session (saved) constraint, if any
if sconstr then
   if sconstr.name~=data.constr then
      if selconstr then
         sconstr=selconstr
         if sconstr then
            sconstr.name=data.constr
         end
      end
   end
end


local notsaved
if sconstr then
   notsaved = true
else
   notsaved = false
   if selconstr then
      sconstr=selconstr
   else -- Create default vals
      sconstr={
         roles={"user"},
         urls={},
         methods=allmethods
      }
   end
end


session.constr=sconstr
if data.constr then
   sconstr.name = data.constr
elseif not sconstr.name then
   sconstr.name = ""
end

-- Take care of all submit buttons, except the Constraint Submit Button
if request:method() == "POST" then
   local methods={}
   for _,v in ipairs(allmethods) do
      if data[v] then tinsert(methods,v) end
   end
   sconstr.methods=methods
   if data.pathsub then
      if data.pathsub == "Delete" then
         if data.pathlist then
            local u=list2tab(sconstr.urls)
            u[data.pathlist]=nil
            sconstr.urls=tab2list(u)
         end
      elseif data.newpath then
         local p=data.newpath:gsub("/*%**$","")
         tinsert(sconstr.urls,p.."/*")
      end
   elseif data.delrolesub then
      local r = list2tab(sconstr.roles)
      for _,v in ipairs(data.croles) do
         r[v]=nil
      end
      sconstr.roles = tab2list(r)
   elseif data.addrolesub then
      local r = list2tab(sconstr.roles)
      for _,v in ipairs(data.aroles) do
         r[v]=true
      end
      sconstr.roles = tab2list(r)
   elseif data.savesub then
      if data.savesub == "Clear" then
         session.constr=nil
         sendredirect()
         return
      elseif data.constr then
         -- Zero methods means ALL methods.
         -- We do not use this logic here. We want zero methods to
         -- disable the constriant.
         if #methods == 0 then
            tinsert(methods,"DISABLED")
         end
         constrs[data.constr]=sconstr
         session.constr=nil
         bd.db.drvcnstr.save({v=constrs})
         sendredirect2constr()
         return
      end
   end
   if selconstr then
      sendredirect2constr()
   else
      sendredirect()
   end
   return
end

 
headerdata=[[
<link href="style.css" rel="stylesheet" type="text/css" />
<style>
.talignr {text-align:right;}
select,textarea {width:130px;margin:3px}
fieldset { float:left;}
table, tr, td {margin:0;padding:0}
td { vertical-align: top; }
.hiddenc {display:none}
#advancedaccess, #basicaccess { width:150px; }
</style>
<script type="text/javascript"  src="wfsconstr.js"></script>
]]
 
title="Menu: Administrator Panel: WFS Constraints"
response:include".header.lsp"

?>

<div id="panel">
<?lsp emitHelpBut("wfsconstr") ?>
<form method="POST">
<input type="hidden" id="basepath" name="basepath" value="<?lsp=session.constrBasepath?>" />
<p>
  <b>Web File Server Constraint Name:</b>
  <?lsp= selconstr and data.constr or fmt('<input type="text" name="constr" value="%s"/>',sconstr.name) ?>
</p>

<table>

<td>
<fieldset>
  <legend><b> Constraints </b></legend>
    <select id="constr" name="constr" size="30">
      <?lsp emitOptions(constrnames,nil,data.constr) ?>
    </select>
    <br/>
    <input id="gotoconstr" type="submit" name="constrsub" value="Goto" />
    <input id="deleteconstr" type="submit" name="constrsub" value="Delete" <?lsp=selconstr and '' or ' disabled="disabled"'?>/>
</fieldset>
</td>


<td>
<table>
<tr><td>
  <fieldset style="width:430px">
    <legend><b> Path </b></legend>
      <select name="pathlist" size="11" style="width:95%">
      <?lsp emitOptions(sconstr.urls) ?>
      </select>
      <br/>
      <input type="submit" id="pathdel" name="pathsub" value="Delete" />
      <br/>
      <input type="text" style="width:300px" id="newpath" name="newpath" value=""/>
      <input type="submit" id="pathsub" name="pathsub" value="Add" />
      <input type="submit" name="pathsub" id="browse" value="Browse" class="hiddenc" />
  </fieldset>
</td></tr>
<tr><td>
  <table style="margin-top:10px;width:100%"><tr>
  <td>
    <fieldset id="advancedaccess">
      <legend><b> Access </b></legend>
      <table>
        <tr><td><?lsp local m = list2tab(sconstr.methods)
                      emitCheckbox("GET",m.GET) ?></td><td>GET</td></tr>
        <tr><td><?lsp emitCheckbox("POST",m.POST) ?></td><td>POST</td></tr>
        <tr><td><?lsp emitCheckbox("PUT",m.PUT) ?></td><td>PUT</td></tr>
        <tr><td><?lsp emitCheckbox("PROPFIND",m.PROPFIND) ?></td><td>PROPFIND</td></tr>
        <tr><td><?lsp emitCheckbox("PROPPATCH",m.PROPPATCH) ?></td><td>PROPPATCH</td></tr>
        <tr><td><?lsp emitCheckbox("MKCOL",m.MKCOL) ?></td><td>MKCOL</td></tr>
        <tr><td><?lsp emitCheckbox("DELETE",m.DELETE) ?></td><td>DELETE</td></tr>
        <tr><td><?lsp emitCheckbox("MOVE",m.MOVE) ?></td><td>MOVE</td></tr>
        <tr><td><?lsp emitCheckbox("COPY",m.COPY) ?></td><td>COPY</td></tr>
      </table>
      <button type="button" class="hiddenc">Basic Options</button>
    </fieldset>
    <fieldset id="basicaccess" class="hiddenc">
      <legend><b> Access </b></legend>
      <table>
        <tr><td><?lsp emitCheckbox("readaccess", true) ?></td><td>Read Access</td></tr>
        <tr><td><?lsp emitCheckbox("writeaccess", true) ?></td><td>Write Access</td></tr>
      </table>
      <button type="button" id="advancedop">Advanced Options</button>
    </fieldset>
  </td>
  <td style="width:100%">
    <table style="float:right"><tr><td>
    <fieldset >
      <legend><b> Constraint's Roles </b></legend>
      <select <?lsp=#sconstr.roles==0 and 'id="superuser"' or ''?> name="croles" multiple="multiple" size="12">
        <?lsp emitOptions(sconstr.roles) ?>
      </select>
    </fieldset>
    </td></tr><tr><td>
    <input type="submit" name="delrolesub" value="Delete Roles" />
    </td></tr></table>
  </td>
  </tr></table>
</td></tr>
</table>
</td>


<td>
<table><tr><td>
<fieldset><legend><b> All Roles </b></legend>
   <select name="aroles" multiple="multiple" size="30">
   <?lsp emitOptions(aroles,sconstr.roles) ?>
</select></fieldset>
</td></tr><tr><td>
<input type="submit" name="addrolesub" value="Add Roles" />
</td></tr></table>
</td>


</table>


<hr/>
<input type="submit" name="savesub" value="Save" />
<?lsp if notsaved then ?>
<input type="submit" name="savesub" value="Clear" />
<?lsp end ?>
</form>
</div>

<?lsp response:include".footer.lsp" ?>
